from __future__ import annotations

from textual.app import ComposeResult
from textual.binding import Binding

from spiel.screens.screen import SpielScreen
from spiel.widgets.footer import Footer
from spiel.widgets.minislides import MiniSlides


class DeckScreen(SpielScreen):
    BINDINGS = [
        Binding("right", "next_slide", "Go to next slide."),
        Binding("left", "prev_slide", "Go to previous slide."),
        Binding("down", "next_row", "Go to next row of slides."),
        Binding("up", "prev_row", "Go to previous row of slides."),
        Binding(
            "escape,enter", "switch_screen('slide')", "Go to Slide view with the selected slide."
        ),
    ]

    def compose(self) -> ComposeResult:
        yield MiniSlides()
        yield Footer()



from __future__ import annotations

import sys
from time import monotonic

from rich.box import HEAVY
from rich.console import RenderableType
from rich.errors import NotRenderableError
from rich.panel import Panel
from rich.protocol import is_renderable
from rich.style import Style
from rich.traceback import Traceback
from textual.reactive import reactive

import spiel
from spiel.exceptions import SpielException
from spiel.triggers import Triggers
from spiel.widgets.widget import SpielWidget


class SlideWidget(SpielWidget):
    triggers: Triggers = reactive(Triggers.new)  # type: ignore[assignment,arg-type]

    def on_mount(self) -> None:
        super().on_mount()

        self.set_interval(1 / 60, self.update_triggers)

    def update_triggers(self) -> None:
        self.triggers = Triggers(now=monotonic(), _times=self.triggers._times)

    def render(self) -> RenderableType:
        try:
            self.remove_class("error")
            r = self.current_slide.render(triggers=self.triggers)
            if is_renderable(r):
                return r
            else:
                raise NotRenderableError(f"object {r!r} is not renderable")
        except Exception:
            self.add_class("error")
            et, ev, tr = sys.exc_info()
            if et is None or ev is None or tr is None:
                raise SpielException("Expected to be handling an exception, but wasn't.")
            return Panel(
                Traceback.from_exception(
                    exc_type=et,
                    exc_value=ev,
                    traceback=tr,
                    suppress=(spiel,),
                ),
                title="Slide content failed to render",
                border_style=Style(bold=True, color="red1"),
                box=HEAVY,
            )



#!/usr/bin/env python

import inspect
import shutil
import socket
from collections.abc import Callable, Iterator
from datetime import datetime
from math import cos, floor, pi
from pathlib import Path
from textwrap import dedent

from click import edit
from rich.align import Align
from rich.box import HEAVY, SQUARE
from rich.color import Color, blend_rgb
from rich.console import Group, RenderableType
from rich.layout import Layout
from rich.markdown import Markdown
from rich.padding import Padding
from rich.panel import Panel
from rich.style import Style
from rich.syntax import Syntax
from rich.text import Text

from spiel import Slide, SuspendType, Triggers, present
from spiel.deck import Deck
from spiel.renderables.image import Image

deck = Deck(name=f"Spiel Demo Deck")

SPIEL = "[Spiel](https://github.com/JoshKarpel/spiel)"
RICH = "[Rich](https://rich.readthedocs.io/)"
IPYTHON = "[IPython](https://ipython.readthedocs.io)"
WSL = "[Windows Subsystem for Linux](https://docs.microsoft.com/en-us/windows/wsl/)"

THIS_FILE = Path(__file__).resolve()
THIS_DIR = THIS_FILE.parent


def pad_markdown(markup: str) -> RenderableType:
    return Padding(Markdown(dedent(markup), justify="center"), pad=(0, 5))


@deck.slide(title="What is Spiel?")
def what() -> RenderableType:
    upper_left = pad_markdown(
        f"""\
        ## What is Spiel?

        {SPIEL} is a framework for building and presenting richly-styled presentations in your terminal using Python.

        Spiel uses {RICH} to render slide content.
        Anything you can display with Rich, you can display with Spiel (plus some other things)!

        Use your right `→` and left `←` arrows keys to go forwards and backwards through the deck.
        Press `ctrl-c` to exit.

        Press `?` at any time to see the help screen, which describes all of the built-in actions you can take.

        To get a copy of the source code for this deck, use the `spiel demo copy` command.
        """
    )

    upper_right = pad_markdown(
        """\
        ## Why use Spiel?

        It's fun!

        It's weird!

        Why not?

        Maybe you shouldn't.

        Honestly, it's unclear whether it's a good idea.

        There's always [Powerpoint](https://youtu.be/uNjxe8ShM-8)!
        """
    )

    lower_left = pad_markdown(
        f"""\
        ## Contributing

        Please report bugs via [GitHub Issues](https://github.com/JoshKarpel/spiel/issues).

        If you have ideas about how Spiel can be improved,
        or you have a cool deck to show off,
        please post to [GitHub Discussions](https://github.com/JoshKarpel/spiel/discussions).
        """
    )

    lower_right = pad_markdown(
        f"""\
        ## Inspirations

        Brandon Rhodes' [PyCon 2017](https://youtu.be/66P5FMkWoVU) and [North Bay Python 2017](https://youtu.be/rrMnmLyYjU8) talks.

        David Beazley's [Lambda Calculus from the Ground Up](https://youtu.be/pkCLMl0e_0k) tutorial at PyCon 2019.

        LaTeX's [Beamer](https://ctan.org/pkg/beamer) document class.
        """
    )

    root = Layout()
    upper = Layout()
    lower = Layout()

    upper.split_row(
        Layout(upper_left),
        Layout(upper_right),
    )
    lower.split_row(
        Layout(lower_left),
        Layout(lower_right),
    )
    root.split_column(upper, lower)

    return root


def make_code_panel_from_object(obj: type | Callable[..., object]) -> RenderableType:
    lines, line_number = inspect.getsourcelines(obj)
    return make_code_panel(line_number, lines)


def make_code_panel(line_number: int, lines: list[str], title: str | None = None) -> RenderableType:
    return Align.center(
        Panel(
            Syntax(
                "".join(lines),
                lexer="python",
                line_numbers=True,
                start_line=line_number,
            ),
            title=title,
            box=SQUARE,
            border_style=Style(dim=True),
            height=len(lines) + 2,
            expand=False,
        )
    )


@deck.slide(title="Decks and Slides")
def code() -> RenderableType:
    markup = f"""\
        ## Decks are made of Slides

        Here's the code for `Deck` and `Slide`!

        The source code is pulled directly from the definitions via [inspect.getsource](https://docs.python.org/3/library/inspect.html#inspect.getsource).

        ({RICH} supports syntax highlighting, so {SPIEL} does too!)
        """
    root = Layout()
    upper = Layout(pad_markdown(markup), size=len(markup.split("\n")) + 1)
    lower = Layout()
    root.split_column(upper, lower)

    lower.split_row(
        Layout(make_code_panel_from_object(Deck)),
        Layout(make_code_panel_from_object(Slide)),
    )

    return root


@deck.slide(title="Dynamic Content")
def dynamic() -> RenderableType:
    width = shutil.get_terminal_size().columns
    width_limit = 80

    home = Path.home()
    home_dir_contents = list(home.iterdir())

    return Group(
        Align.center(
            pad_markdown(
                f"""\
                ## Slides can have dynamic content!

                Since slides are created using normal Python code,
                any output you can imagine producing via Python can make it into your slides.

                Here are some examples:
                """
            ),
        ),
        Align.center(
            Panel(
                Text(
                    f"Your terminal is {width} cells wide (try resizing it or adjusting your font size!)"
                    if width > width_limit
                    else f"Your terminal is only {width} cells wide! Get a bigger monitor!",
                    style=Style(color="green1" if width > width_limit else "red"),
                    justify="center",
                )
            ),
        ),
        Align.center(
            Panel(
                Text.from_markup(
                    f"The local timezone on this computer ([bold]{socket.gethostname()}[/bold]) is [bold]{datetime.now().astimezone().tzinfo}[/bold]",
                    style="bright_cyan",
                    justify="center",
                )
            ),
        ),
        Align.center(
            Panel(
                Text(
                    f"There are {len([f for f in home_dir_contents if f.is_file()])} files and {len([f for f in home_dir_contents if f.is_dir()])} directories in {home}",
                    style=Style(color="yellow"),
                    justify="center",
                )
            ),
        ),
    )


@deck.slide(title="Triggers")
def triggers(triggers: Triggers) -> RenderableType:
    info = pad_markdown(
        f"""\
        ## Triggers

        Triggers are a mechanism for making dynamic content that depends on *relative* time.

        Triggers can be used to implement effects like fades, motion, and other "animations".

        Each slide is triggered once when it starts being displayed.

        You can trigger it again (as many times as you'd like) by pressing `t`.
        You can reset the trigger state by pressing `r`.

        This slide has been triggered {len(triggers)} times.

        It was last triggered {triggers.time_since_last_trigger:.2f} seconds ago.
        """
    )

    white = Color.parse("bright_white")
    black = Color.parse("black")
    red = Color.parse("bright_red")
    green = Color.parse("bright_green")

    fade_time = 3

    lines = [
        Text(
            f"Triggered at {time - triggers[0]:.3f}!",
            style=Style(
                color=(
                    Color.from_triplet(
                        blend_rgb(
                            black.get_truecolor(),
                            white.get_truecolor(),
                            cross_fade=min((triggers.now - time) / fade_time, 1),
                        )
                    )
                )
            ),
        )
        for time in triggers
    ]

    fun = Padding(
        Align.center(
            Panel(
                Text("\n", justify="center").join(lines),
                border_style=Style(
                    color=Color.from_triplet(
                        blend_rgb(
                            green.get_truecolor(),
                            red.get_truecolor(),
                            cross_fade=min(triggers.time_since_last_trigger / fade_time, 1),
                        )
                    ),
                ),
                title="Trigger Tracker",
            )
        ),
        pad=(1, 0),
    )

    return Group(info, fun)


def make_reveals(triggers: Triggers) -> Iterator[RenderableType]:
    return triggers.take(
        Align.center(r)
        for r in [
            Text("Reveal 1", style=Style(color="black", bgcolor="#E40303")),
            Text("Reveal 2", style=Style(color="black", bgcolor="#FF8C00")),
            Text("Reveal 3", style=Style(color="black", bgcolor="#FFED00")),
            Text("Reveal 4", style=Style(color="black", bgcolor="#008026")),
            Text("Reveal 5", style=Style(color="black", bgcolor="#24408E")),
            Text("Reveal 6", style=Style(color="black", bgcolor="#732982")),
        ]
    )


@deck.slide(title="Triggers: Reveals")
def bullets(triggers: Triggers) -> RenderableType:
    info_upper = pad_markdown(
        f"""\
        ## Triggers: Reveals

        Triggers can be useful even without considering their tracking of relative time.

        We can track the number of times the slide has been triggered to gradually
        reveal content.

        `{Triggers.take.__qualname__}` makes this straightforward:
        """
    )

    info_lower = pad_markdown(
        f"""\
        Trigger this slide (press `t`) a few times to reveal some content.
        Press `r` to hide the content again (by resetting the trigger state).
        """
    )

    return Group(
        info_upper,
        Padding(make_code_panel_from_object(make_reveals), pad=(0, 0, 1, 0)),
        info_lower,
        *make_reveals(triggers),
    )


def make_bullet(triggers: Triggers) -> RenderableType:
    bounce_period = 10
    width = 50
    half_width = width // 2

    bounce_time = triggers.time_since_first_trigger % bounce_period
    bounce_character = "⁍" if bounce_time < (1 / 2) * bounce_period else "⁌"
    bounce_position = floor(half_width * cos(2 * pi * bounce_time / bounce_period))
    before = half_width + bounce_position

    bullet = Padding(
        bounce_character,
        pad=(0, before, 0, (half_width - bounce_position - 1)),
    )

    return Align.center(
        Panel(bullet, title="Bouncing Bullet", padding=0),
        vertical="middle",
    )


@deck.slide(title="Triggers: Animations")
def bouncing_bullet(triggers: Triggers) -> RenderableType:
    info = pad_markdown(
        f"""\
        ## Triggers: Animations

        Here's an example of how triggers can be used to build
        more complex animations.

        The position and facing direction of the bullet are calculated deterministically
        based on the time since the first trigger time (the automatic one
        from when the slide starts being presented).

        There is no state stored in the slide function itself.
        The apparent motion is based on {SPIEL} evaluating the
        slide content function with different `Trigger` values.
        """
    )

    return Group(info, make_bullet(triggers), make_code_panel_from_object(make_bullet))


@deck.slide(title="Views")
def grid() -> RenderableType:
    return pad_markdown(
        """\
        ## Deck View

        Try pressing `d` to go into "deck" view.
        You can move between slides in deck view using your arrow keys (right `→`, left `←`, up `↑`, and down `↓`).

        Press `enter` or `escape` to go back to "slide" view (this view),
        on the currently-selected slide.
        """
    )


@deck.slide(title="Displaying Images")
def image() -> RenderableType:
    markup = f"""\
        ## Images

        {SPIEL} can display images... sort of!

        Spiel includes an `Image` widget that can render images by interpolating pixel values.

        If you see big chunks of constant color instead of smooth gradients, your terminal is probably not configured for "truecolor" mode.
        If your terminal supports truecolor (it probably does), try setting the environment variable `COLORTERM` to `truecolor`.

        For example, for `bash`, you could add

        `export COLORTERM=truecolor`

        to your `.bashrc` file, then restart your shell.
        """

    image_path = THIS_DIR / "tree.jpg"
    root = Layout()
    root.split_row(
        Layout(pad_markdown(markup)),
        Layout(
            Panel.fit(
                Align.center(Image.from_file(image_path), vertical="middle"),
                subtitle=image_path.name,
                box=HEAVY,
                padding=0,
            )
        ),
    )

    return root


@deck.slide(title="Watch Mode")
def watch() -> RenderableType:
    return pad_markdown(
        f"""\
        ## Developing a Deck

        {SPIEL} will reload your deck as you edit it to make development easier.

        The reload is triggered whenever any files under the path passed to the
        `--watch` argument of `spiel present` changes.
        That path defaults to the parent directory of the deck file.
        """
    )


def edit_this_file(suspend: SuspendType) -> None:
    with suspend():
        edit(filename=__file__)


@deck.slide(
    title="Bindings",
    bindings={
        "e": edit_this_file,
    },
)
def bindings() -> RenderableType:
    edit_this_file_source_lines, line_number = inspect.getsourcelines(edit_this_file)
    this_slide_source_lines, _ = inspect.getsourcelines(bindings)
    source_lines = [
        *edit_this_file_source_lines,
        "\n",  # blank line between the two functions
        *this_slide_source_lines[:7],  # up through the slide's def
        "    ...",  # replace the slide body with a "..."
    ]
    code = make_code_panel(line_number, source_lines, title=THIS_FILE.name)

    info_upper = pad_markdown(
        f"""\
        ## Custom Per-Slide Key Bindings

        Custom keybindings can be added on a per-slide basis using the `bindings` argument of `@slide`,
        which takes a mapping of key names to callables to call when that key is pressed.


        If the callable takes an argument named `suspend`,
        it will be passed a function that, when used as a context manager,
        suspends {SPIEL} while inside the `with` block.

        A binding has been registered on this slide that suspends {SPIEL}
        and opens your `$EDITOR` on the demo deck's Python file:
        """
    )

    info_lower = pad_markdown(
        f"""\
        Try pressing `e`!

        Due to reloading, any changes you make will be reflected in the
        presentation you're seeing right now.
        """
    )

    return Group(info_upper, code, info_lower)


class DemoRenderFailure(Exception):
    pass


@deck.slide(title="Render Failure")
def failure() -> RenderableType:
    raise DemoRenderFailure(
        f"""Woops!

        An exception was raised while rendering this slide.

        When this happens, Spiel will display the stack trace to help you debug the problem.

        Deck reloading will still happen, so you can fix the error without stopping Spiel.

        This error display will also be shown if you return slide content that can't be rendered by Rich.
        """
    )


if __name__ == "__main__":
    present(__file__)



from __future__ import annotations

from collections.abc import Callable, Iterator, Mapping, Sequence
from dataclasses import dataclass, field
from typing import overload

from spiel.slide import Content, Slide


@dataclass
class Deck(Sequence[Slide]):
    """
    Represents a "deck" of "slides": a presentation.
    """

    name: str
    """The name of the `Deck`/presentation, which will be displayed in the footer."""

    _slides: list[Slide] = field(default_factory=list)

    def slide(
        self,
        title: str = "",
        bindings: Mapping[str, Callable[..., None]] | None = None,
    ) -> Callable[[Content], Content]:
        """
        A decorator that creates a new slide in the deck,
        with the decorated function as the `Slide`'s `content`.

        Args:
            title: The title to display for the slide.
            bindings: A mapping of
                [keys](https://textual.textualize.io/guide/input/#key)
                to callables to be executed when those keys are pressed,
                when on this slide.
        """

        def slideify(content: Content) -> Content:
            self.add_slides(
                Slide(
                    title=title,
                    content=content,
                    bindings=bindings or {},
                )
            )
            return content

        return slideify

    def add_slides(self, *slides: Slide) -> None:
        """
        Add `Slide`s to a `Deck`.

        This function is primarily useful when adding multiple slides at once,
        probably generated programmatically.
        If adding a single slide, prefer the [`Deck.slide`][spiel.Deck.slide] decorator.

        Args:
            *slides: The `Slide`s to add.
        """
        self._slides.extend(slides)

    def __len__(self) -> int:
        return len(self._slides)

    @overload
    def __getitem__(self, item: int) -> Slide:
        return self._slides[item]

    @overload
    def __getitem__(self, item: slice) -> Sequence[Slide]:
        return self._slides[item]

    def __getitem__(self, item: int | slice) -> Slide | Sequence[Slide]:
        return self._slides[item]

    def __iter__(self) -> Iterator[Slide]:
        yield from self._slides

    def __contains__(self, item: object) -> bool:
        return item in self._slides



from __future__ import annotations

from rich.console import RenderableType
from rich.padding import Padding
from rich.table import Column, Table
from rich.text import Text
from textual.binding import Binding

from spiel.widgets.widget import SpielWidget


class AppBindingsTableWidget(SpielWidget):
    DEFAULT_CSS = """
    AppBindingsTableWidget {
        width: auto;
        height: auto;
    }
    """

    def render(self) -> RenderableType:
        table = Table(
            Column("Key", justify="left"),
            Column("Description", justify="left"),
            title=f"All Views",
        )

        for binding in self.app.BINDINGS:
            if isinstance(binding, Binding):
                table.add_row(binding.key, binding.description)
            else:
                raise TypeError(f"{binding} on {self.app} needs to be a {Binding.__name__}")

        return Padding(table, pad=1)


class ScreenBindingsTableWidget(SpielWidget):
    DEFAULT_CSS = """
    ScreenBindingsTableWidget {
        width: auto;
        height: auto;
    }
    """

    def render(self) -> RenderableType:
        if self.id is None:
            return Text("")

        screen = self.app.get_screen(self.id)
        table = Table(
            Column("Key", justify="left"),
            Column("Description", justify="left"),
            title=f"{self.id.title()} View",
        )

        for binding in screen.BINDINGS:
            if isinstance(binding, Binding):
                table.add_row(binding.key, binding.description)
            else:
                raise TypeError(f"{binding} on {screen} needs to be a {Binding.__name__}")

        return Padding(table, pad=1)



from spiel.app import SuspendType, present
from spiel.constants import __version__
from spiel.deck import Deck
from spiel.slide import Slide
from spiel.triggers import Triggers

__all__ = ["present", "SuspendType", "__version__", "Deck", "Slide", "Triggers"]



from __future__ import annotations

from collections.abc import Iterable
from itertools import zip_longest
from typing import TypeVar

T = TypeVar("T")


def filter_join(separator: str, items: Iterable[object | None]) -> str:
    return separator.join(map(str, filter(None, items)))


def clamp(value: int, lower: int, upper: int) -> int:
    if lower > upper:
        raise ValueError(
            f"Upper bound ({upper}) for clamp must be greater than lower bound ({lower})."
        )
    return max(min(value, upper), lower)


def chunks(iterable: Iterable[T], n: int, fill_value: T | None = None) -> Iterable[Iterable[T]]:
    args = [iter(iterable)] * n
    return zip_longest(*args, fillvalue=fill_value)



import sys
from importlib import metadata
from pathlib import Path

PACKAGE_NAME = "spiel"
__version__ = metadata.version(PACKAGE_NAME)
__rich_version__ = metadata.version("rich")
__textual_version__ = metadata.version("textual")
__python_version__ = ".".join(map(str, sys.version_info))

DECK = "deck"

PACKAGE_DIR = Path(__file__).resolve().parent
DEMO_DIR = PACKAGE_DIR / "demo"
DEMO_FILE = PACKAGE_DIR / "demo" / "demo.py"

FOOTER_TIME_FORMAT = "%Y-%m-%d %I:%M %p"
RELOAD_MESSAGE_TIME_FORMAT = "%I:%M:%S %p"



from __future__ import annotations

from textual.app import ComposeResult
from textual.binding import Binding
from textual.containers import Container

from spiel.screens.screen import SpielScreen
from spiel.widgets.bindings import AppBindingsTableWidget, ScreenBindingsTableWidget
from spiel.widgets.footer import Footer


class HelpScreen(SpielScreen):
    DEFAULT_CSS = """
    .h-section {
        layout: horizontal;
        height: auto;
        align: center top;
        content-align: center top;
    }
    """

    BINDINGS = [
        Binding("escape,enter", "pop_screen", "Return to the previous view."),
    ]

    def compose(self) -> ComposeResult:
        yield Container(
            AppBindingsTableWidget(),
            classes="h-section",
        )
        yield Container(
            ScreenBindingsTableWidget(id="slide"),
            ScreenBindingsTableWidget(id="deck"),
            classes="h-section",
        )
        yield Container(
            ScreenBindingsTableWidget(id="help"),
            classes="h-section",
        )
        yield Footer()



from __future__ import annotations

from collections.abc import Iterable, Sequence
from dataclasses import dataclass
from functools import cached_property
from itertools import islice
from time import monotonic
from typing import Iterator, TypeVar, overload

T = TypeVar("T")


@dataclass(frozen=True)
class Triggers(Sequence[float]):
    """
    Provides information to [`Slide.content`][spiel.Slide.content] about the current slide's "trigger state".

    `Triggers` is a [`Sequence`][collections.abc.Sequence] of times
    (produced by [`time.monotonic`][time.monotonic])
    that the current slide was triggered at.
    Note that the slide will be triggered once when it starts being displayed,
    so the first trigger time will be the time when the slide started being displayed.
    """

    now: float
    """
    The time that the slide content is being rendered at.
    Use this is as a single consistent value to base relative times on.
    """

    _times: tuple[float, ...]

    def __post_init__(self) -> None:
        if not self._times:
            raise ValueError("times must not be empty")

        if self.now < self._times[-1]:
            raise ValueError(f"now {self.now} must be later than the last time {self._times[-1]}")

    @classmethod
    def new(self) -> Triggers:
        now = monotonic()
        return Triggers(now=now, _times=(now,))

    def __len__(self) -> int:
        return len(self._times)

    @overload
    def __getitem__(self, item: int) -> float:
        return self._times[item]

    @overload
    def __getitem__(self, item: slice) -> Sequence[float]:
        return self._times[item]

    def __getitem__(self, item: int | slice) -> float | Sequence[float]:
        return self._times[item]

    def __iter__(self) -> Iterator[float]:
        return iter(self._times)

    def __contains__(self, item: object) -> bool:
        return item in self._times

    @cached_property
    def time_since_last_trigger(self) -> float:
        """The elapsed time since the most recent trigger."""
        return self.now - self._times[-1]

    @cached_property
    def time_since_first_trigger(self) -> float:
        """
        The elapsed time since the first trigger,
        which is equivalent to the time since the slide started being displayed.
        """
        return self.now - self._times[0]

    @cached_property
    def triggered(self) -> bool:
        """
        Returns whether the slide has been *manually* triggered
        (i.e., this ignores the initial trigger from when the slide starts being displayed).
        """
        return len(self) > 1

    def take(self, iter: Iterable[T], offset: int = 1) -> Iterator[T]:
        """
        Takes elements from the iterable `iter`
        equal to the number of times in the `Triggers` minus the offset.

        Args:
            iter: The iterable to take elements from.
            offset: This `offset` will be subtracted from the number of triggers,
                reducing the number of elements that will be returned.
                It defaults to `1` to ignore the automatic trigger from when the
                slide starts being shown.

        Returns:
            An iterator over the first `len(self) - offset` elements of `iter`.
        """
        return islice(iter, len(self) - offset)



from __future__ import annotations

import asyncio
import code
import datetime
import importlib.util
import os
import sys
from asyncio import wait
from contextlib import contextmanager, redirect_stderr, redirect_stdout
from functools import cached_property, partial
from pathlib import Path
from time import monotonic
from typing import Callable, ContextManager, Iterator

from rich.style import Style
from rich.text import Text
from textual import log
from textual.app import App
from textual.binding import Binding
from textual.events import Resize
from textual.reactive import reactive
from watchfiles import awatch

from spiel.constants import DECK, RELOAD_MESSAGE_TIME_FORMAT
from spiel.deck import Deck
from spiel.exceptions import NoDeckFound
from spiel.screens.deck import DeckScreen
from spiel.screens.help import HelpScreen
from spiel.screens.slide import SlideScreen
from spiel.triggers import Triggers
from spiel.utils import clamp
from spiel.widgets.slide import SlideWidget


def load_deck(path: Path) -> Deck:
    module_name = "__deck"
    spec = importlib.util.spec_from_file_location(module_name, path)

    if spec is None:
        raise NoDeckFound(f"{path.resolve()} does not appear to be an importable Python module.")

    module = importlib.util.module_from_spec(spec)
    sys.modules[module_name] = module

    loader = spec.loader
    assert loader is not None
    loader.exec_module(module)

    try:
        deck = getattr(module, DECK)
    except AttributeError:
        raise NoDeckFound(f"The module at {path} does not have an attribute named {DECK}.")

    if not isinstance(deck, Deck):
        raise NoDeckFound(
            f"The module at {path} has an attribute named {DECK}, but it is a {type(deck).__name__}, not a {Deck.__name__}."
        )

    return deck


SuspendType = Callable[[], ContextManager[None]]


class SpielApp(App[None]):
    CSS_PATH = "spiel.css"
    BINDINGS = [
        Binding("d", "switch_screen('deck')", "Go to the Deck view."),
        Binding("question_mark", "push_screen('help')", "Go to the Help view."),
        Binding("i", "repl", "Switch to the REPL."),
        Binding("p", "screenshot", "Take a screenshot."),
    ]

    deck = reactive(Deck(name="New Deck"))
    current_slide_idx = reactive(0)
    message = reactive(Text(""))

    def __init__(
        self,
        deck_path: Path,
        watch_path: Path | None = None,
        show_messages: bool = True,
        fixed_time: datetime.datetime | None = None,
    ):
        super().__init__()

        self.deck_path = deck_path
        self.watch_path = watch_path

        self.show_messages = show_messages
        self.fixed_time = fixed_time

    async def on_mount(self) -> None:
        self.deck = load_deck(self.deck_path)
        self.reloader = asyncio.create_task(self.reload())

        await self.install_screen(SlideScreen(), name="slide")
        await self.install_screen(DeckScreen(), name="deck")
        await self.install_screen(HelpScreen(), name="help")
        await self.push_screen("slide")

    async def reload(self) -> None:
        if self.watch_path is None:
            return

        log(f"Watching {self.watch_path} for changes")
        async for changes in awatch(self.watch_path):
            change_msg = "\n  ".join([""] + [f"{k.raw_str()}: {v}" for k, v in changes])
            log(f"Reloading deck from {self.deck_path} due to detected file changes:{change_msg}")
            try:
                self.deck = load_deck(self.deck_path)
                self.current_slide_idx = clamp(self.current_slide_idx, 0, len(self.deck))
                self.set_message_temporarily(
                    Text(
                        f"Reloaded deck at {datetime.datetime.now().strftime(RELOAD_MESSAGE_TIME_FORMAT)}",
                        style=Style(dim=True),
                    ),
                    delay=10,
                )
            except Exception as e:
                self.set_message_temporarily(
                    Text(
                        f"Failed to reload deck at {datetime.datetime.now().strftime(RELOAD_MESSAGE_TIME_FORMAT)} due to: {e}",
                        style=Style(color="red"),
                    ),
                    delay=10,
                )

    def on_resize(self, event: Resize) -> None:
        self.set_message_temporarily(
            message=Text(f"Screen resized to {event.size}", style=Style(dim=True)), delay=2
        )

    def set_message_temporarily(self, message: Text, delay: float) -> None:
        if not self.show_messages:
            return

        self.message = message

        def clear() -> None:
            if self.message is message:
                self.message = Text("")

        self.set_timer(delay, clear)

    def action_next_slide(self) -> None:
        self.current_slide_idx = clamp(self.current_slide_idx + 1, 0, len(self.deck) - 1)

    def action_prev_slide(self) -> None:
        self.current_slide_idx = clamp(self.current_slide_idx - 1, 0, len(self.deck) - 1)

    def action_next_row(self) -> None:
        self.current_slide_idx = clamp(
            self.current_slide_idx + self.deck_grid_width, 0, len(self.deck) - 1
        )

    def action_prev_row(self) -> None:
        self.current_slide_idx = clamp(
            self.current_slide_idx - self.deck_grid_width, 0, len(self.deck) - 1
        )

    def watch_deck(self, new_deck: Deck) -> None:
        self.title = new_deck.name

    def watch_current_slide_idx(self, new_current_slide_idx: int) -> None:
        self.query_one(SlideWidget).triggers = Triggers.new()
        self.sub_title = self.deck[new_current_slide_idx].title

    def action_trigger(self) -> None:
        now = monotonic()
        slide_widget = self.query_one(SlideWidget)
        slide_widget.triggers = Triggers(now=now, _times=(*slide_widget.triggers._times, now))

    def action_reset_trigger(self) -> None:
        slide_widget = self.query_one(SlideWidget)
        slide_widget.triggers = Triggers.new()

    @cached_property
    def repl(self) -> Callable[[], None]:
        # Lazily enable readline support
        import readline  # nopycln: import

        self.console.clear()  # clear the console the first time we go into the repl
        sys.stdout.flush()

        repl = code.InteractiveConsole()
        return partial(repl.interact, banner="", exitmsg="")

    def action_repl(self) -> None:
        with self.suspend():
            self.repl()

    async def action_quit(self) -> None:
        self.reloader.cancel()
        await wait([self.reloader], timeout=1)

        await super().action_quit()

    @contextmanager
    def suspend(self) -> Iterator[None]:
        driver = self._driver

        if driver is not None:
            driver.stop_application_mode()
            driver.exit_event.clear()  # type: ignore[attr-defined]
            with redirect_stdout(sys.__stdout__), redirect_stderr(sys.__stderr__):
                yield
            driver.start_application_mode()

    @property
    def deck_grid_width(self) -> int:
        return max(self.size.width // 35, 1)


def present(deck_path: Path | str, watch_path: Path | str | None = None) -> None:
    """
    Present the deck defined in the given `deck_path`.

    Args:
        deck_path: The file to look for a deck in.
        watch_path: When filesystem changes are detected below this path (recursively), reload the deck from the `deck_path`.
            If `None` (the default), use the parent directory of the `deck_path`.
    """
    os.environ["TEXTUAL"] = ",".join(sorted({"debug", "devtools"}))

    deck_path = Path(deck_path).resolve()
    watch_path = Path(watch_path or deck_path.parent).resolve()

    SpielApp(deck_path=deck_path, watch_path=watch_path).run()



from __future__ import annotations

from datetime import datetime

from rich.console import Group, RenderableType
from rich.rule import Rule
from rich.style import Style
from rich.table import Column, Table
from rich.text import Text
from textual.reactive import reactive

from spiel.constants import FOOTER_TIME_FORMAT
from spiel.widgets.widget import SpielWidget


class Footer(SpielWidget):
    DEFAULT_CSS = """
    Footer {
        color: $text;
        dock: bottom;
        height: 2;
    }
    """

    now: datetime = reactive(datetime.now)  # type: ignore[arg-type,assignment]

    def on_mount(self) -> None:
        super().on_mount()

        self.update_now()
        self.set_interval(1 / 60, self.update_now)

    def update_now(self) -> None:
        self.now = self.app.fixed_time or datetime.now()

    @property
    def longest_slide_number_length(self) -> int:
        num_slides = len(self.app.deck)
        return len(str(num_slides))

    def render(self) -> RenderableType:
        grid = Table.grid(
            Column(style=Style(dim=True), justify="left"),
            Column(style=Style(bold=True), justify="center"),
            Column(style=Style(dim=True), justify="right"),
            expand=True,
            padding=1,
        )
        grid.add_row(
            Text(f"{self.app.deck.name} | {self.app.deck[self.app.current_slide_idx].title}"),
            self.app.message,
            Text(
                f"{self.now.strftime(FOOTER_TIME_FORMAT)}   [{self.app.current_slide_idx + 1:>0{self.longest_slide_number_length}d} / {len(self.app.deck)}]"
            ),
        )
        return Group(Rule(style=Style(dim=True)), grid)



from __future__ import annotations

from rich.console import Console, ConsoleOptions, RenderResult
from rich.style import Style
from rich.table import Column, Table
from rich.text import Text

from spiel.constants import (
    PACKAGE_NAME,
    __python_version__,
    __rich_version__,
    __textual_version__,
    __version__,
)


class DebugTable:
    def __rich_console__(self, console: Console, options: ConsoleOptions) -> RenderResult:
        table = Table(
            Column(justify="right"),
            Column(justify="left"),
            show_header=False,
            title="Debug Information",
        )

        table.add_row(f"{PACKAGE_NAME.capitalize()} Version", __version__)
        table.add_row("Rich Version", __rich_version__)
        table.add_row("Textual Version", __textual_version__)
        table.add_row("Python Version", __python_version__)

        table.add_section()

        table.add_row(
            "Color System",
            Text(
                console.color_system or "unknown",
                style=Style(color="red" if console.color_system != "truecolor" else "green"),
            ),
        )
        table.add_row(
            "Console Dimensions",
            Text(f"{console.width} cells wide, {console.height} cells tall"),
            end_section=True,
        )

        yield table



from __future__ import annotations

from dataclasses import dataclass
from functools import lru_cache
from math import floor
from pathlib import Path
from typing import Iterable, NamedTuple

from PIL import Image as Img
from PIL.Image import Resampling
from rich.color import Color
from rich.console import Console, ConsoleOptions
from rich.segment import Segment
from rich.style import Style

from spiel.utils import chunks


class ImageSize(NamedTuple):
    width: int
    height: int


Pixels = tuple[tuple[int, int, int] | None, ...]


@lru_cache(maxsize=2**8)
def _pixels_to_segments(pixels: Pixels, size: ImageSize) -> list[Segment]:
    line = Segment.line()

    segments = []
    pixel_row_pairs = chunks(chunks(pixels, size.width), 2, fill_value=[None] * size.width)
    for top_pixel_row, bottom_pixel_row in pixel_row_pairs:
        for top_pixel, bottom_pixel in zip(top_pixel_row, bottom_pixel_row):
            # use upper-half-blocks for the top pixel row and the background color for the bottom pixel row
            segments.append(
                Segment(
                    text="▀",
                    style=Style.from_color(
                        color=Color.from_rgb(*top_pixel[:3]) if top_pixel else None,
                        bgcolor=Color.from_rgb(*bottom_pixel[:3]) if bottom_pixel else None,
                    ),
                )
            )
        segments.append(line)

    return list(Segment.simplify(segments))


@lru_cache(maxsize=2**4)
def _load_image(path: Path) -> Image:
    img = Img.open(path)
    img.load()
    return img


@dataclass(frozen=True)
class Image:
    img: Img

    @classmethod
    def from_file(cls, path: Path) -> Image:
        return cls(img=_load_image(path))

    def _determine_size(self, options: ConsoleOptions) -> ImageSize:
        width, height = self.img.size

        # multiply the max height by 2, because we're going to print 2 "pixels" per row
        max_height = options.height * 2 if options.height else None
        if max_height:
            width, height = width * max_height / self.img.height, max_height

        if width > options.max_width:
            width, height = options.max_width, height * options.max_width / width

        return ImageSize(floor(width), floor(height))

    def _resize(self, size: ImageSize) -> Img:
        return self.img.resize(
            size=size,
            resample=Resampling.LANCZOS,
        )

    def __rich_console__(self, console: Console, options: ConsoleOptions) -> Iterable[Segment]:
        size = self._determine_size(options)
        resized = self._resize(size)
        pixels = tuple(resized.getdata())
        yield from _pixels_to_segments(pixels, size)



from __future__ import annotations

import inspect
from dataclasses import dataclass, field
from typing import Callable, Mapping

from rich.console import RenderableType
from rich.text import Text

from spiel.triggers import Triggers

TRIGGERS = "triggers"

Content = Callable[..., RenderableType]


@dataclass
class Slide:
    """
    Represents a single slide in the presentation.
    """

    title: str = ""
    """The title of the `Slide`, which will be displayed in the footer."""

    content: Content = lambda: Text()
    """
    A callable that is invoked by Spiel to display the slide's content.

    The function may optionally take arguments with these names:

    - `trigger`: The current [`Trigger`][spiel.Triggers] state, for use in animations.
    """

    bindings: Mapping[str, Callable[..., None]] = field(default_factory=dict)

    def render(self, triggers: Triggers) -> RenderableType:
        signature = inspect.signature(self.content)

        kwargs: dict[str, object] = {}
        if TRIGGERS in signature.parameters:
            kwargs[TRIGGERS] = triggers

        return self.content(**kwargs)



from __future__ import annotations

from itertools import islice
from math import ceil

from rich.console import RenderableType
from rich.layout import Layout
from rich.panel import Panel
from rich.style import Style
from rich.text import Text

from spiel.triggers import Triggers
from spiel.utils import clamp
from spiel.widgets.widget import SpielWidget


class MiniSlides(SpielWidget):
    def render(self) -> RenderableType:
        grid_width = self.app.deck_grid_width
        row_of_current_slide = self.app.current_slide_idx // grid_width
        num_rows = ceil(len(self.app.deck) / grid_width)
        start_row = clamp(
            value=row_of_current_slide - (grid_width // 2),
            lower=0,
            upper=max(num_rows - grid_width, 0),
        )
        start_slide_idx = grid_width * start_row
        slides = islice(enumerate(self.app.deck), start_slide_idx, None)

        rows = [Layout(name=str(r)) for r in range(grid_width)]
        cols = [[Layout(name=f"{r}-{c}") for c in range(grid_width)] for r, _ in enumerate(rows)]

        root = Layout()
        root.split_column(*rows)

        for row, layouts in zip(rows, cols):
            row.split_row(*layouts)

            for layout in layouts:
                slide_idx, slide = next(slides, (None, None))
                if slide_idx is None or slide is None:
                    layout.update("")
                else:
                    is_active_slide = slide_idx == self.app.current_slide_idx

                    try:
                        content = slide.render(triggers=Triggers.new())
                        border_style = Style(
                            color="bright_cyan" if is_active_slide else None,
                            dim=not is_active_slide,
                        )
                    except Exception as e:
                        content = Text(
                            f"Failed to render slide {slide_idx + 1} due to:\n{e}",
                            style=Style(color="red"),
                        )
                        border_style = Style(
                            color="red1",
                            dim=not is_active_slide,
                        )

                    layout.update(
                        Panel(
                            content,
                            title=" | ".join((str(slide_idx + 1), slide.title)),
                            border_style=border_style,
                        )
                    )

        return root



from __future__ import annotations

from typing import TYPE_CHECKING

from textual.reactive import watch
from textual.widget import Widget

from spiel.slide import Slide

if TYPE_CHECKING:
    from spiel.app import SpielApp


class SpielWidget(Widget):
    app: "SpielApp"

    def on_mount(self) -> None:
        watch(self.app, "deck", self.r)
        watch(self.app, "current_slide_idx", self.r)
        watch(self.app, "message", self.r)

    def r(self, _: object) -> None:
        self.refresh()

    @property
    def current_slide(self) -> Slide:
        return self.app.deck[self.app.current_slide_idx]



from __future__ import annotations

import inspect

from textual.app import ComposeResult
from textual.binding import Binding
from textual.events import Key

from spiel.screens.screen import SpielScreen
from spiel.widgets.footer import Footer
from spiel.widgets.slide import SlideWidget

SUSPEND = "suspend"


class SlideScreen(SpielScreen):
    BINDINGS = [
        Binding("right", "next_slide", "Go to next slide."),
        Binding("left", "prev_slide", "Go to previous slide."),
        Binding("t", "trigger", "Trigger the current slide."),
        Binding("r", "reset_trigger", "Reset trigger state."),
    ]

    def compose(self) -> ComposeResult:
        yield SlideWidget()
        yield Footer()

    def on_key(self, event: Key) -> None:
        slide = self.app.deck[self.app.current_slide_idx]
        bind = slide.bindings.get(event.key)

        if callable(bind):
            signature = inspect.signature(bind)

            kwargs: dict[str, object] = {}
            if SUSPEND in signature.parameters:
                kwargs[SUSPEND] = self.app.suspend

            bind(**kwargs)



import shutil
from pathlib import Path
from textwrap import dedent
from typing import Optional

import typer.rich_utils as ru
from click.exceptions import Exit
from rich.console import Console
from rich.style import Style
from rich.syntax import Syntax
from rich.text import Text
from typer import Argument, Option, Typer

from spiel.app import present
from spiel.constants import DEMO_DIR, DEMO_FILE, PACKAGE_DIR, PACKAGE_NAME, __version__
from spiel.renderables.debug import DebugTable

ru.STYLE_HELPTEXT = ""

console = Console()


cli = Typer(
    name=PACKAGE_NAME,
    no_args_is_help=True,
    rich_markup_mode="rich",
    help=dedent(
        f"""\
        Display [italic yellow]Rich[/italic yellow]ly-styled presentations using your terminal.

        To see what {PACKAGE_NAME.capitalize()} can do, take a look at the demo deck:

            $ spiel demo present

        A {PACKAGE_NAME.capitalize()} presentation (a "[italic green]deck[/italic green] of slides")
        is defined programmatically using a Python script.
        """
    ),
)


@cli.command(name="present")
def _present(
    path: Path = Argument(
        ...,
        dir_okay=False,
        exists=True,
        readable=True,
        help="The path to the slide deck file.",
    ),
    watch: Optional[Path] = Option(
        default=None,
        help="When filesystem changes are detected below this path (recursively), reload the deck from the deck path. Defaults to the parent directory of the deck path.",
    ),
) -> None:
    """
    Present a deck.
    """
    present(deck_path=path, watch_path=watch)


@cli.command()
def init(
    path: Path = Argument(
        ...,
        writable=True,
        resolve_path=True,
        help="The path to create a new deck script at.",
    )
) -> None:
    """
    Create a new deck script at the given path from a basic template.

    This is a good starting point if you already know what you want to do.
    If you're not so sure, consider taking a look at the demo deck to see what's possible:
        $ spiel demo --help
    """
    console = Console()

    if path.exists():
        console.print(
            Text(f"Error: {path} already exists, refusing to overwrite.", style=Style(color="red"))
        )
        raise Exit(code=1)

    name = path.stem.replace("_", " ").title()

    try:
        path.parent.mkdir(parents=True, exist_ok=True)
    except Exception as e:
        console.print(
            Text(
                f"Error: was not able to ensure that the parent directory {path.parent} exists due to: {e}.",
                style=Style(color="red"),
            )
        )
        raise Exit(code=1)

    try:
        path.write_text(
            dedent(
                f"""\
                from textwrap import dedent
                from spiel import Deck


                deck = Deck(name="{name}")

                @deck.slide(title="Title")
                def title():
                    markup = dedent(
                        \"""\\
                        # {name}
                        This is your title slide!
                        \"""
                    )
                    return Markdown(markup, justify="center")
                """
            )
        )
    except Exception as e:
        console.print(
            Text(
                f"Error: was not able to write template to {path} due to: {e}",
                style=Style(color="red"),
            )
        )
        raise Exit(code=1)

    console.print(Text(f"Wrote deck template to {path}", style=Style(color="green")))


demo = Typer(
    name="demo",
    no_args_is_help=True,
    rich_markup_mode="rich",
    help=dedent(
        """\
        Use the demonstration deck (present it, display source, etc.).
        """
    ),
)
cli.add_typer(demo)


@demo.command(name="present")
def present_demo() -> None:
    """
    Present the demo deck.
    """
    present(deck_path=DEMO_FILE, watch_path=PACKAGE_DIR)


@demo.command()
def source() -> None:
    """
    Display the source code for the demo deck in your PAGER.
    """
    console = Console()

    with console.pager(styles=True):
        console.print(Syntax(DEMO_FILE.read_text(encoding="utf-8"), lexer="python"))


@demo.command()
def copy(
    path: Path = Argument(
        default=...,
        exists=False,
        writable=True,
        help="The path to copy the demo deck source code and assets to.",
    )
) -> None:
    """
    Copy the demo deck source code and assets to a new directory.

    If you're looking for a more stripped-down starting point, try the init command:
        $ spiel init --help
    """
    console = Console()

    if path.exists():
        console.print(Text(f"Error: {path} already exists!", style=Style(color="red")))
        raise Exit(code=1)

    try:
        shutil.copytree(DEMO_DIR, path)
    except Exception as e:
        console.print(Text(f"Failed to copy demo deck directory: {e}", style=Style(color="red")))
        raise Exit(code=1)

    console.print(
        Text(f"Wrote demo deck source code and assets to {path}", style=Style(color="green"))
    )


@cli.command()
def version(
    plain: bool = Option(
        default=False,
        help=f"Print only {PACKAGE_NAME}'s version.",
    )
) -> None:
    """
    Display version and debugging information.
    """

    if plain:
        console.print(__version__, style=Style())
    else:
        console.print(DebugTable())



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['spiel', 'spiel.demo', 'spiel.renderables', 'spiel.screens', 'spiel.widgets']

package_data = \
{'': ['*']}

install_requires = \
['more-itertools>=9',
 'pillow>=8',
 'rich>=12',
 'textual==0.4.0',
 'typer>=0.6',
 'watchfiles>=0.18']

entry_points = \
{'console_scripts': ['spiel = spiel.cli:cli']}

setup_kwargs = {
    'name': 'spiel',
    'version': '0.4.5',
    'description': 'A framework for building and presenting richly-styled presentations in your terminal using Python.',
    'long_description': '# Spiel\n\n[![PyPI](https://img.shields.io/pypi/v/spiel)](https://pypi.org/project/spiel)\n[![PyPI - License](https://img.shields.io/pypi/l/spiel)](https://pypi.org/project/spiel)\n[![Docs](https://img.shields.io/badge/docs-exist-brightgreen)](https://www.spiel.how)\n\n[![pre-commit.ci status](https://results.pre-commit.ci/badge/github/JoshKarpel/spiel/main.svg)](https://results.pre-commit.ci/latest/github/JoshKarpel/spiel/main)\n[![codecov](https://codecov.io/gh/JoshKarpel/spiel/branch/main/graph/badge.svg?token=2sjP4V0AfY)](https://codecov.io/gh/JoshKarpel/spiel)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n\n[![GitHub issues](https://img.shields.io/github/issues/JoshKarpel/spiel)](https://github.com/JoshKarpel/spiel/issues)\n[![GitHub pull requests](https://img.shields.io/github/issues-pr/JoshKarpel/spiel)](https://github.com/JoshKarpel/spiel/pulls)\n\n[Spiel](https://dictionary.cambridge.org/us/dictionary/english/spiel) is a framework for building and presenting\n[richly-styled](https://github.com/Textualize/rich) presentations in your terminal using Python.\n\nTo see what Spiel can do without installing it, you can view the demonstration deck in a container:\n```bash\n$ docker run -it --rm ghcr.io/joshkarpel/spiel\n```\nAlternatively, install Spiel (`pip install spiel`) and run this command to view the demonstration deck:\n```bash\n$ spiel demo present\n```\n\n![The first slide of the demo deck](https://raw.githubusercontent.com/JoshKarpel/spiel/main/docs/assets/demo.svg)\n![The demo deck in "deck view"](https://raw.githubusercontent.com/JoshKarpel/spiel/main/docs/assets/deck.svg)\n\n## Quick Start\n\nIf you want to jump right in,\ninstall Spiel (`pip install spiel`),\ncreate a file called `deck.py`,\nand copy this code into it:\n```python\nfrom rich.console import RenderableType\n\nfrom spiel import Deck, present\n\ndeck = Deck(name="Your Deck Name")\n\n\n@deck.slide(title="Slide 1 Title")\ndef slide_1() -> RenderableType:\n    return "Your content here!"\n\n\nif __name__ == "__main__":\n    present(__file__)\n```\n\nThat is the most basic Spiel presentation you can make.\nTo present the deck, run `python deck.py`.\nYou should see:\n\n![Barebones slide](https://raw.githubusercontent.com/JoshKarpel/spiel/main/docs/assets/quickstart_basic.svg)\n\nCheck out the [Quick Start tutorial](https://www.spiel.how/quickstart) to continue!\n\n## Documentation\n\nTo learn more about Spiel, take a look at the [documentation](https://www.spiel.how).\n\n## Contributing\n\nIf you\'re interested in contributing to Spiel, check out the [Contributing Guide](https://www.spiel.how/contributing/).\n',
    'author': 'JoshKarpel',
    'author_email': 'josh.karpel@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/JoshKarpel/spiel',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4',
}


setup(**setup_kwargs)
